/*
 * Austin---Astonishing Universal Search Tree Interface Novelty
 * Copyright (C) 2000 Kaz Kylheku <kaz@ashi.footprints.net>
 *
 * Free Software License:
 *
 * All rights are reserved by the author, with the following exceptions:
 * Permission is granted to freely reproduce and distribute this software,
 * possibly in exchange for a fee, provided that this copyright notice appears
 * intact. Permission is also granted to adapt this software to produce
 * derivative works, as long as the modified versions carry this copyright
 * notice and additional notices stating that the work has been modified.
 * This source code may be translated into executable form and incorporated
 * into proprietary software; there is no requirement for such software to
 * contain a copyright notice related to this source.
 *
 * $Id: testlib.c,v 1.1 1999/11/25 00:46:26 kaz Exp $
 * $Name: austin_0_1 $
 */

#include "udict.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>

typedef char input_t[256];

static int tokenize(char *string, ...)
{
    char **tokptr; 
    va_list arglist;
    int tokcount = 0;

    va_start(arglist, string);
    tokptr = va_arg(arglist, char **);
    while (tokptr) {
	while (*string && isspace((unsigned char) *string))
	    string++;
	if (!*string)
	    break;
	*tokptr = string;
	while (*string && !isspace((unsigned char) *string))
	    string++;
	tokptr = va_arg(arglist, char **);
	tokcount++;
	if (!*string)
	    break;
	*string++ = 0;
    }
    va_end(arglist);

    return tokcount;
}

static int comparef(const void *key1, const void *key2)
{
    return strcmp(key1, key2);
}

static char *dupstring(char *str)
{
    int sz = strlen(str) + 1;
    char *new = malloc(sz);
    if (new)
	memcpy(new, str, sz);
    return new;
}

static udict_node_t *new_node(void *c)
{
    static udict_node_t few[5];
    static int count;

    if (count < 5)
	return few + count++;

    return NULL;
}

static void del_node(void *c, udict_node_t *n)
{
}

int main(void)
{
    input_t in;
    udict_t *d = udict_create(UDICT_BST, UDICT_COUNT_T_MAX, comparef);
    udict_node_t *dn;
    char *tok1, *tok2, *val;
    const char *key;
    int prompt = 0;

    char *help =
	"a <key> <val>          add value to dictionary\n"
	"d <key>                delete value from dictionary\n"
	"l <key>                lookup value in dictionary\n"
	"( <key>                lookup lower bound\n"
	") <key>                lookup upper bound\n"
	"k                      allow duplicate keys\n"
	"c                      show number of entries\n"
	"t                      dump whole dictionary in sort order\n"
	"p                      turn prompt on\n"
	"s                      switch to non-functioning allocator\n"
	"m <num>                morph sturcture to algorithm <num>\n"
	"q                      quit";

    if (!d)
	puts("udict_create failed");

    for (;;) {
	if (prompt)
	    putchar('>');
	fflush(stdout);

	if (!fgets(in, sizeof(input_t), stdin))
	    break;

	switch(in[0]) {
	    case '?':
		puts(help);
		break;
	    case 'a':
		if (tokenize(in+1, &tok1, &tok2, (char **) 0) != 2) {
		    puts("what?");
		    break;
		}
		key = dupstring(tok1);
		val = dupstring(tok2);

		if (!key || !val) {
		    puts("out of memory");
		    free((void *) key);
		    free(val);
		}

		if (!udict_alloc_insert(d, key, val)) {
		    puts("udict_alloc_insert failed");
		    free((void *) key);
		    free(val);
		    break;
		}
		break;
	    case 'd':
		if (tokenize(in+1, &tok1, (char **) 0) != 1) {
		    puts("what?");
		    break;
		}
		dn = udict_lookup(d, tok1);
		if (!dn) {
		    puts("udict_lookup failed");
		    break;
		}
		val = udict_node_getdata(dn);
		key = udict_node_getkey(dn);
		udict_delete_free(d, dn);

		free(val);
		free((void *) key);
		break;
	    case 'l':
	    case '(':
	    case ')':
		if (tokenize(in+1, &tok1, (char **) 0) != 1) {
		    puts("what?");
		    break;
		}
		dn = 0;
		switch (in[0]) {
		case 'l':
		    dn = udict_lookup(d, tok1);
		    break;
		case '(':
		    dn = udict_lower_bound(d, tok1);
		    break;
		case ')':
		    dn = udict_upper_bound(d, tok1);
		    break;
		}
		if (!dn) {
		    puts("lookup failed");
		    break;
		}
		val = udict_node_getdata(dn);
		puts(val);
		break;
	    case 'k':
		udict_allow_dupes(d);
		break;
	    case 'c':
		printf("%lu\n", (unsigned long) udict_count(d));
		break;
	    case 't':
		for (dn = udict_first(d); dn; dn = udict_next(d, dn)) {
		    printf("%s\t%s\n", (char *) udict_node_getkey(dn),
			    (char *) udict_node_getdata(dn));
		}
		break;
	    case 'q':
		exit(0);
		break;
	    case 'm':
		if (tokenize(in+1, &tok1, (char **) 0) != 1) {
		    puts("what?");
		    break;
		}
		udict_convert_to(d, atoi(tok1));
		break;
	    case '\0':
		break;
	    case 'p':
		prompt = 1;
		break;
	    case 's':
		udict_set_allocator(d, new_node, del_node, NULL);
		break;
	    default:
		putchar('?');
		putchar('\n');
		break;
	}
    }

    return 0;
}

